from snakemake.shell import shell
import platform
import pathlib
import os
import argparse
import json

log = snakemake.log_fmt_shell(stdout=True, stderr=True)
oc_version = snakemake.params.get('oc_version', None)
oc_install_needed = False
try:
    import cravat.admin_util as au
    if oc_version is not None and oc_version != au.get_current_package_version():
        oc_install_needed = True
except ModuleNotFoundError:
    oc_install_needed = True
if oc_install_needed:
    if oc_version is not None:
        cmd = f'pip install open-cravat=={oc_version}'
    else:
        cmd = f'pip install open-cravat'
    shell(cmd)
    import cravat.admin_util as au
sysconf = au.get_system_conf()
root_dir = snakemake.config.get('root_dir', None)
if root_dir is None:
    if 'modules_dir' in sysconf:
        root_dir = os.path.dirname(sysconf['modules_dir'])
    else:
        pl = platform.platform()
        if pl.endswith('Windows'):
            root_dir = 'C:/open-cravat'
        elif pl.endswith('Darwin'):
            root_dir = '/home/Shared/open-cravat'
        elif pl.contains('Linux'):
            root_dir = os.path.join(pathlib.Path.home(), 'open-cravat')
        else:
            root_dir = os.path.join(os.getcwd(), 'open-cravat')
modules_dir = sysconf.get('modules_dir', os.path.join(root_dir, 'modules'))
logs_dir = sysconf.get('logs_dir', os.path.join(root_dir, 'logs'))
jobs_dir = sysconf.get('jobs_dir', os.path.join(root_dir, 'jobs'))
conf_dir = sysconf.get('conf_dir', os.path.join(root_dir, 'conf'))
if os.path.exists(root_dir) == False:
    os.mkdir(root_dir)
if os.path.exists(modules_dir) == False:
    os.mkdir(modules_dir)
    print(f'oc modules are at [{modules_dir}]')
if os.path.exists(logs_dir) == False:
    os.mkdir(logs_dir)
    sysconf['logs_dir'] = logs_dir
    print(f'oc server jobs are at [{jobs_dir}]')
if os.path.exists(jobs_dir) == False:
    os.mkdir(jobs_dir)
    sysconf['jobs_dir'] = jobs_dir
    print(f'oc server logs are at [{logs_dir}]')
if os.path.exists(conf_dir) == False:
    os.mkdir(conf_dir)
    sysconf['conf_dir'] = conf_dir
    print(f'oc conf is at [{conf_dir}]')
if 'gui_port' in snakemake.config:
    gui_port = snakemake.config['gui_port']
    sysconf['gui_port'] = gui_port
au.write_system_conf_file(sysconf)
if 'modules_dir' not in sysconf:
    cmd = f'oc config md {modules_dir}'
    shell(cmd)
    print(f'Installing system modules...')
    cmd = f'oc module install-base'
    #shell(cmd)
annotators = snakemake.params.get('annotators', None)
if annotators is not None:
    for module_name in annotators:
        version = annotators[module_name]
        local_module_info = au.get_local_module_info(module_name)
        if local_module_info is None or local_module_info.version != version:
            print(f'Installing module {module_name}:{version}...')
            cmd = f'oc module install {module_name} -v {version} -y'
            shell(cmd)
        else:
            print(f'{module_name}:{version} already exists. Skipping installation.')
